﻿#include "loginwin.h"
#include "ui_loginwin.h"
#include "utils.h"
#include "sqlitedb.h"
#include "globalParams.h"
#include <QSettings>
#include <qt_windows.h>
#include <dwmapi.h>
#include <windows.h>
#include <windowsx.h>
#include <cmath>
#include <QGraphicsDropShadowEffect>
#include <QPainter>
#include <QPainterPath>
#include <QMessageBox>
#include <QRegularExpression>
#include <QRegularExpressionMatch>
#include <QMessageBox>
#include <QDebug>

#ifdef defined(_MSC_VER)
#pragma comment(lib, "user32.lib")
#pragma comment(lib,"dwmapi.lib")
#endif

LoginWin::LoginWin(QWidget *parent) :
    QWidget(parent),
    ui(new Ui::LoginWin)
{
    ui->setupUi(this);
    this->setWindowFlags(Qt::FramelessWindowHint | Qt::WindowMinMaxButtonsHint);
    this->setPalette(QPalette(QColor(240,240,240)));
    ui->titleBar->setPalette(QPalette(QColor(240,240,240)));
    this->setAutoFillBackground(true);

    int w = this->width();
    int h = this->height();
    this->setMinimumSize(w,h);
    this->setMaximumSize(w,h);

#ifdef defined(_MSC_VER)
    /*
    此行代码可以带回Aero效果，同时也带回了标题栏和边框,在nativeEvent()会再次去掉标题栏
    主要的关键是WS_THICKFRAME，可以实现窗口停靠边缘自动改变大小功能和Aero效果
    */
    HWND hwnd = (HWND)this->winId();
    DWORD style = ::GetWindowLong(hwnd, GWL_STYLE);
    ::SetWindowLong(hwnd, GWL_STYLE, style | WS_THICKFRAME);

    //保留一个像素的边框宽度，否则系统不会绘制边框阴影
    const MARGINS shadow = { 1, 1, 1, 1 };
    DwmExtendFrameIntoClientArea(HWND(winId()), &shadow);
#endif

    connect(ui->titleBar,SIGNAL(windowCloseSignal()),this,SLOT(windowCloseHandle()),Qt::QueuedConnection);

    readCfg();
}

LoginWin::~LoginWin()
{
    delete ui;
}

void LoginWin::windowCloseHandle()
{
    QSettings setting(globalParams::configPath,QSettings::IniFormat);
    QString check;
    if(ui->ckbRemember->isChecked()){
        check = "1";
    }else{
        check = "0";
    }
    setting.setValue("users/remember",check);
    close();
}

void LoginWin::on_btnLogin_clicked()
{
    if(ui->cmbName->currentText().isEmpty()){
        QMessageBox::warning(this,"警告","请输入用户名！",QMessageBox::Ok,QMessageBox::NoButton);
        return;
    }
    if(ui->lePassword->text().isEmpty()){
        QMessageBox::warning(this,"警告","请输入密码！",QMessageBox::Ok,QMessageBox::NoButton);
        return;
    }
    QString curName = ui->cmbName->currentText().trimmed();
    QString curPassword = ui->lePassword->text().trimmed();
    QMap<QString,QString> map;
    map["ID"] = curName;
    char base64[256] = {'\0'};
    int len = 0;
    base64_encode((const unsigned char *)curPassword.toStdString().c_str(),strlen(curPassword.toStdString().c_str()),(unsigned char *)base64,&len);
    map["PASSWORD"] = QString::fromLocal8Bit(base64,len);
    QVector<QMap<QString, QString>> result = SQLITEDB::getInstance().getRecord("USERS",map);
    if(result.size()<=0){
         QMessageBox::warning(this,"警告",QString("用户名或密码错误！"),QMessageBox::NoButton);
        return;
    }
    nameMap[curName] = curPassword;
    globalParams::curUserId = result[0].value("ID");
    globalParams::curUserType = result[0].value("USER_TYPE");
    saveCfg();
    emit loginSignal();
    windowCloseHandle();
}

bool LoginWin::nativeEvent(const QByteArray &eventType, void *message, long *result)
{
    MSG* param = static_cast<MSG*>(message);

    switch (param->message)
    {
    case WM_NCCALCSIZE: {
        /*
        此消息用于处理非客户区域，比如边框的绘制
        返回false,就是按系统的默认处理，如果返回true,而不做任何绘制，则非客户区域
        就不会被绘制，就相当于没有绘制非客户区域，所以就会看不到非客户区域的效果
        */
        return true;
    }
    case WM_NCHITTEST:
    {

        const LONG border_width = 3;
        RECT winrect;
        GetWindowRect(HWND(winId()), &winrect);

        long x = GET_X_LPARAM(param->lParam);
        long y = GET_Y_LPARAM(param->lParam);

        /*
        只用这种办法设置动态改变窗口大小比手动通过鼠标事件效果好，可以
        避免闪烁问题
        */
        //bottom left corner
        if (x >= winrect.left && x < winrect.left + border_width &&
            y < winrect.bottom && y >= winrect.bottom - border_width)
        {
            *result = HTBOTTOMLEFT;
            return true;
        }
        //bottom right corner
        if (x < winrect.right && x >= winrect.right - border_width &&
            y < winrect.bottom && y >= winrect.bottom - border_width)
        {
            *result = HTBOTTOMRIGHT;
            return true;
        }
        //top left corner
        if (x >= winrect.left && x < winrect.left + border_width &&
            y >= winrect.top && y < winrect.top + border_width)
        {
            *result = HTTOPLEFT;
            return true;
        }
        //top right corner
        if (x < winrect.right && x >= winrect.right - border_width &&
            y >= winrect.top && y < winrect.top + border_width)
        {
            *result = HTTOPRIGHT;
            return true;
        }
        //left border
        if (x >= winrect.left && x < winrect.left + border_width)
        {
            *result = HTLEFT;
            return true;
        }
        //right border
        if (x < winrect.right && x >= winrect.right - border_width)
        {
            *result = HTRIGHT;
            return true;
        }

        //bottom border
        if (y < winrect.bottom && y >= winrect.bottom - border_width)
        {
            *result = HTBOTTOM;
            return true;
        }
        //top border
        if (y >= winrect.top && y < winrect.top + border_width)
        {
            *result = HTTOP;
            return true;
        }

        return QWidget::nativeEvent(eventType, message, result);
    }
    }

    return QWidget::nativeEvent(eventType, message, result);
}

void LoginWin::readCfg()
{
    QSettings setting(globalParams::configPath,QSettings::IniFormat);
    QString names = setting.value("users/names").toString().trimmed();
    QStringList nameList = names.split(";");

    for(int i=0;i<nameList.size();i++){
        QString nameAndPassword = nameList[i];
        QStringList list = nameAndPassword.split(":");
        if(list.size()==2){
            QString name = list[0];
            QString base64 = list[1];
            char password[256] = {'\0'};
            int len = 0;
            base64_decode((const unsigned char *)base64.toStdString().c_str(),strlen(base64.toStdString().c_str()),(unsigned char *)password,&len);
            nameMap[name] = QString::fromLocal8Bit(password,len);
        }
    }

    foreach (const QString &name, nameMap.keys()) {
        if(!name.isEmpty()){
            ui->cmbName->addItem(name);
        }
    }
    QString curName = setting.value("users/current").toString().trimmed();
    if(!curName.isEmpty()){
        ui->cmbName->setCurrentText(curName);
    }

    QString check = setting.value("users/remember").toString();
    if(check.toInt()==1){
        ui->ckbRemember->setChecked(true);
    }else{
        ui->ckbRemember->setChecked(false);
    }
}

void LoginWin::saveCfg()
{
    QSettings setting(globalParams::configPath,QSettings::IniFormat);

    QString names;
    QString curname = ui->cmbName->currentText();
    foreach (const QString &name, nameMap.keys()) {
        if(!ui->ckbRemember->isChecked() && name==curname){
            continue;
        }
        if(!name.isEmpty()){
            QString password = nameMap.value(name,"");
            char base64[256] = {'\0'};
            int len = 0;
            base64_encode((const unsigned char *)password.toStdString().c_str(),strlen(password.toStdString().c_str()),(unsigned char *)base64,&len);
            names += QString("%1:%2;").arg(name,base64);
        }
    }
    setting.setValue("users/names",names);
}

void LoginWin::on_cmbName_currentTextChanged(const QString &arg1)
{
    ui->lePassword->setText(nameMap.value(arg1,""));
}

