#include "mainwindow.h"
#include "ui_mainwindow.h"
#include "globalParams.h"
#include "msgInputWin.h"
#include "testThread.h"
#include <QSerialPortInfo>
#include <QGroupBox>
#include <QHBoxLayout>
#include <QPushButton>
#include <QRegularExpression>
#include <QDebug>

MainWindow::MainWindow(QWidget *parent)
    : QMainWindow(parent)
    , ui(new Ui::MainWindow)
{
    ui->setupUi(this);
    this->setWindowTitle("北斗用户机测试软件");
    //this->setPalette(QPalette(QColor(255,255,255)));
    //this->setPalette(QPalette(QColor(225,225,225)));
    ui->tabWidget->setCurrentIndex(0);

    serialPortInit();

    //状态栏显示当前用户
    QGroupBox *userBox = new QGroupBox();
    userBox->setStyleSheet("QGroupBox{border:none;}");
    QHBoxLayout *userLayout = new QHBoxLayout(userBox);
    userLayout->setMargin(0);
    QLabel *lbUserSpaceLeft = new QLabel();
    QLabel *lbUserSpaceRight = new QLabel();
    lbUserSpaceLeft->setFixedWidth(10);
    lbUserSpaceRight->setFixedWidth(10);
    QLabel *lbUserId = new QLabel("当前用户：");
    lbCurUserId = new QLabel();
    lbCurUserId->setText(globalParams::curUserId);
    userLayout->addWidget(lbUserSpaceLeft);
    userLayout->addWidget(lbUserId);
    userLayout->addWidget(lbCurUserId);
    userLayout->addWidget(lbUserSpaceRight);
    ui->statusbar->addWidget(userBox);

    //显示测试进度
    QGroupBox *progressBox = new QGroupBox();
    progressBox->setStyleSheet("QGroupBox{border:none;}");
    QHBoxLayout *progressLayout = new QHBoxLayout(progressBox);
    progressLayout->setMargin(0);
    QLabel *lbProgressSpaceLeft = new QLabel();
    QLabel *lbProgressSpaceMid1 = new QLabel();
    QLabel *lbProgressSpaceMid2 = new QLabel();
    QLabel *lbProgressSpaceMid3 = new QLabel();
    QLabel *lbProgressSpaceRight = new QLabel();
    lbProgressSpaceLeft->setFixedWidth(5);
    lbProgressSpaceMid1->setFixedWidth(5);
    lbProgressSpaceMid2->setFixedWidth(5);
    lbProgressSpaceMid3->setFixedWidth(5);
    lbProgressSpaceRight->setFixedWidth(5);

    QLabel *lbCurTest = new QLabel("当前测试项：");
    QLabel *lbTestTime = new QLabel("测试时间：");
    lbCurTestItem = new QLabel("");
    lbCurTestTimeCounts = new QLabel("");

    progressLayout->addWidget(lbProgressSpaceLeft);
    progressLayout->addWidget(lbCurTest);
    progressLayout->addWidget(lbCurTestItem);
    progressLayout->addWidget(lbProgressSpaceMid1);
    progressLayout->addWidget(lbTestTime);
    progressLayout->addWidget(lbCurTestTimeCounts);
    progressLayout->addWidget(lbProgressSpaceRight);

    ui->statusbar->addWidget(progressBox);

    //状态栏显示接收比特数
    QGroupBox *byteBox = new QGroupBox();
    byteBox->setStyleSheet("QGroupBox{border:none;}");
    QHBoxLayout *byteLayout = new QHBoxLayout(byteBox);
    byteLayout->setMargin(0);
    QLabel *lbByteSpaceLeft = new QLabel();
    QLabel *lbByteSpaceMid1 = new QLabel();
    QLabel *lbByteSpaceMid2 = new QLabel();
    QLabel *lbByteSpaceMid3 = new QLabel();
    QLabel *lbByteSpaceRight = new QLabel();
    lbByteSpaceLeft->setFixedWidth(5);
    lbByteSpaceMid1->setFixedWidth(5);
    lbByteSpaceMid2->setFixedWidth(5);
    lbByteSpaceMid3->setFixedWidth(5);
    lbByteSpaceRight->setFixedWidth(5);

    QLabel *lbSend = new QLabel("发送：");
    QLabel *lbRecv = new QLabel("接收：");
    QPushButton *btnClean = new QPushButton();
    connect(btnClean,SIGNAL(clicked()),this,SLOT(btnCleanClickedHandle()),Qt::QueuedConnection);
    lbComStatus = new QLabel();
    lbComStatus->setFixedSize(16,16);
    lbComStatus->setPixmap(QPixmap(":/images/circle_gray.png"));
    lbComStatus->setScaledContents(true);
    btnClean->setFixedSize(18,18);
    btnClean->setAutoRepeat(true);
    btnClean->setStyleSheet("QPushButton{background-image:url(:/images/delete_black.png);\
                               background-color:rgb(240,240,240);background-position:center;;border:none}" \
                               "QPushButton:hover{background-color:rgba(233,233,233,150);" \
                                "background-image:url(:/images/delete_red.png);border:none;}");
    lbSendByte = new QLabel("0");
    lbRecvByte = new QLabel("0");
    byteLayout->addWidget(lbByteSpaceLeft);
    byteLayout->addWidget(lbComStatus);
    byteLayout->addWidget(lbByteSpaceMid1);
    byteLayout->addWidget(lbSend);
    byteLayout->addWidget(lbSendByte);
    byteLayout->addWidget(lbByteSpaceMid2);
    byteLayout->addWidget(lbRecv);
    byteLayout->addWidget(lbRecvByte);
    byteLayout->addWidget(lbByteSpaceMid3);
    byteLayout->addWidget(btnClean);
    byteLayout->addWidget(lbByteSpaceRight);
    ui->statusbar->addWidget(byteBox);

    connect(&TestThread::getInstance(),SIGNAL(writeDataSignal(QByteArray)),this,SIGNAL(writeDataSignal(QByteArray)),Qt::QueuedConnection);
    connect(&TestThread::getInstance(),SIGNAL(curTestInfoSignal(QString,int)),this,SLOT(curTestInfoHandle(QString,int)),Qt::QueuedConnection);

    //状态检测
    connect(ui->statusTestWdg,SIGNAL(writeDataSignal(QByteArray)),this,SIGNAL(writeDataSignal(QByteArray)),Qt::QueuedConnection);
    connect(this,SIGNAL(recvSbxSignal(QString)),ui->statusTestWdg,SLOT(recvSbxHandle(QString)),Qt::QueuedConnection);
    connect(this,SIGNAL(recvIciSignal(QString)),ui->statusTestWdg,SLOT(recvIciHandle(QString)),Qt::QueuedConnection);
    connect(this,SIGNAL(recvPidSignal(QString)),ui->statusTestWdg,SLOT(recvPidHandle(QString)),Qt::QueuedConnection);
    connect(this,SIGNAL(recvIidSignal(QString)),ui->statusTestWdg,SLOT(recvIidHandle(QString)),Qt::QueuedConnection);
    connect(this,SIGNAL(recvIcmSignal(QString)),ui->statusTestWdg,SLOT(recvIcmHandle(QString)),Qt::QueuedConnection);
    connect(this,SIGNAL(recvPmuSignal(QString)),ui->statusTestWdg,SLOT(recvPmuHandle(QString)),Qt::QueuedConnection);
}

MainWindow::~MainWindow()
{
    serial->close();
    delete serial;
    delete ui;
}

void MainWindow::receiverSerialDataHandle(const QByteArray &data)
{
    long long totalLen = lbRecvByte->text().toLongLong();
    totalLen += data.size();
    lbRecvByte->setText(QString::number(totalLen));
    try {
        leftData.append(data);
    } catch (...) {
        qDebug()<<"new data can't append to buf";
        return;
    }
    QStringList lines;
    int start = 0;
    for(int i=0;i<leftData.size()-1;i++){
        if((unsigned char)(leftData[i])==0x0D && (unsigned char)(leftData[i+1])==0x0A){
            QByteArray tmp = leftData.mid(start,i+1-start+1);
            start = i+2;
            lines.append(QString::fromLocal8Bit(tmp));
        }
    }
    leftData = leftData.mid(start);
    if(lines.isEmpty() && leftData.size()>4096){
        leftData = leftData.right(4096);
    }
    foreach (const QString &line, lines) {
        QRegularExpression re("\\$[^\\r\\n\\*\\$]+\\*[A-Z0-9]{1,2}\\r{0,1}\\n");
        QRegularExpressionMatch match = re.match(line);
        if(match.hasMatch()){
            QString cmd = match.captured(0);
            if(cmd.startsWith("$BDSBX")){
                emit recvSbxSignal(cmd);
            }else if(cmd.startsWith("$BDICI")){
                emit recvIciSignal(cmd);
            }else if(cmd.startsWith("$BDPID")){
                emit recvPidSignal(cmd);
            }else if(cmd.startsWith("$BDIID")){
                emit recvIidSignal(cmd);
            }else if(cmd.startsWith("$BDICM")){
                emit recvIcmSignal(cmd);
            }else if(cmd.startsWith("$BDPMU")){
                emit recvPmuSignal(cmd);
            }
        }
    }

}

void MainWindow::writeBytesHandle(int bytes)
{
    long long totalLen = lbSendByte->text().toLongLong();
    totalLen += bytes;
    lbSendByte->setText(QString::number(totalLen));
}

void MainWindow::loginHandle()
{
    lbCurUserId->setText(globalParams::curUserId);
    this->show();
}

void MainWindow::curTestInfoHandle(const QString &item, int timeCounts)
{
    lbCurTestItem->setText(item);
    lbCurTestTimeCounts->setText(QString::number(timeCounts));
}

void MainWindow::btnCleanClickedHandle()
{
    lbSendByte->setText("0");
    lbRecvByte->setText("0");
}

void MainWindow::on_tbtnFresh_clicked()
{
    QString curCom = ui->cmbCom->currentText();
    ui->cmbCom->clear();
    QStringList nameList;
    foreach (const QSerialPortInfo &info, QSerialPortInfo::availablePorts())
    {
        nameList.append(info.portName());
    }
    std::sort(nameList.begin(), nameList.end(), [&](QString &s1, QString &s2)
              {
                  if(s1.size()==s2.size()){
                      return s1<s2;
                  }else{
                      return s1.size()<s2.size();
                  }
              });
    ui->cmbCom->addItems(nameList);
    ui->cmbCom->setCurrentText(curCom);
}

void MainWindow::serialPortInit()
{
    serial = new SerialPort();
    connect(serial,SIGNAL(recvSerialDataSignal(QByteArray)),this,SLOT(receiverSerialDataHandle(QByteArray)),Qt::QueuedConnection);
    connect(this,SIGNAL(writeDataSignal(QByteArray)),serial,SLOT(writeData(QByteArray)),Qt::QueuedConnection);
    connect(serial,SIGNAL(writeBytesSignal(int)),this,SLOT(writeBytesHandle(int)),Qt::QueuedConnection);
    connect(serial,SIGNAL(recvSerialDataSignal(QByteArray)),&TestThread::getInstance(),SLOT(receiverSerialDataHandle(QByteArray)),Qt::QueuedConnection);
    QStringList nameList;
    foreach (const QSerialPortInfo &info, QSerialPortInfo::availablePorts())
    {
        nameList.append(info.portName());
    }

    std::sort(nameList.begin(), nameList.end(), [&](QString &s1, QString &s2)
              {
                  if(s1.size()==s2.size()){
                      return s1<s2;
                  }else{
                      return s1.size()<s2.size();
                  }
              });
    ui->cmbCom->addItems(nameList);
}

void MainWindow::on_tbtnConnect_clicked()
{
    if(!bconnected){
        serial->setSerialconfig(ui->cmbCom->currentText(),ui->cmbBaud->currentText().toInt());
        if(serial->open()){
            ui->tbtnConnect->setText("断开");
            ui->tbtnConnect->setStyleSheet("color:red");
            lbComStatus->setPixmap(QPixmap(":/images/circle_green.png"));
            bconnected = true;
            ui->cmbBaud->setEnabled(false);
            ui->cmbCom->setEnabled(false);
            ui->tbtnFresh->setEnabled(false);
        }
    }else{
        ui->tbtnConnect->setText("连接");
        ui->tbtnConnect->setStyleSheet("color:green");
        lbComStatus->setPixmap(QPixmap(":/images/circle_gray.png"));
        bconnected = false;
        serial->close();
        ui->cmbBaud->setEnabled(true);
        ui->cmbCom->setEnabled(true);
        ui->tbtnFresh->setEnabled(true);
    }
}

