#include "msgInputWin.h"
#include "ui_msgInputWin.h"
#include "globalParams.h"
#include <QComboBox>
#include <QSettings>
#include <QDebug>

MsgInputWin::MsgInputWin(QWidget *parent)
    : QWidget(parent)
    , ui(new Ui::MsgInputWin)
{
    ui->setupUi(this);
    msgModel = new QStandardItemModel(this);
    msgSelect = new QItemSelectionModel(msgModel);
    msgModel->setColumnCount(2);
    ui->tableView->setModel(msgModel);
    ui->tableView->setSelectionModel(msgSelect);
    ui->tableView->horizontalHeader()->setVisible(false);
    ui->tableView->verticalHeader()->setVisible(false);
    ui->tableView->horizontalHeader()->setStretchLastSection(true);
    ui->tableView->verticalHeader()->setSectionResizeMode(QHeaderView::Stretch);

    QFont font = ui->tableView->font();
    //font.setPointSize(10);
    ui->tableView->setFont(font);

    QList<QStandardItem *> aItemList;
    QStandardItem *aItem = new QStandardItem("单位");
    aItem->setEditable(false);
    font.setBold(true);
    aItem->setFont(font);
    aItem->setTextAlignment(Qt::AlignCenter);
    aItemList.append(aItem);
    aItem = new QStandardItem("");
    aItemList.append(aItem);
    msgModel->insertRow(msgModel->rowCount(),aItemList);

    aItemList.clear();
    aItem = new QStandardItem("班组");
    aItem->setEditable(false);
    aItem->setFont(font);
    aItem->setTextAlignment(Qt::AlignCenter);
    aItemList.append(aItem);
    aItem = new QStandardItem("");
    aItemList.append(aItem);
    msgModel->insertRow(msgModel->rowCount(),aItemList);

    aItemList.clear();
    aItem = new QStandardItem("车型");
    aItem->setEditable(false);
    aItem->setFont(font);
    aItem->setTextAlignment(Qt::AlignCenter);
    aItemList.append(aItem);
    aItem = new QStandardItem("");
    aItemList.append(aItem);
    msgModel->insertRow(msgModel->rowCount(),aItemList);
    QComboBox *carType = new QComboBox();
    carType->setCursor(Qt::PointingHandCursor);
    carType->setEditable(true);
    ui->tableView->setIndexWidget(msgModel->index(msgModel->rowCount()-1,1),carType);

    aItemList.clear();
    aItem = new QStandardItem("北斗型号");
    aItem->setEditable(false);
    aItem->setFont(font);
    aItem->setTextAlignment(Qt::AlignCenter);
    aItemList.append(aItem);
    aItem = new QStandardItem("");
    aItemList.append(aItem);
    msgModel->insertRow(msgModel->rowCount(),aItemList);

    aItemList.clear();
    aItem = new QStandardItem("北斗序列号");
    aItem->setEditable(false);
    aItem->setFont(font);
    aItem->setTextAlignment(Qt::AlignCenter);
    aItemList.append(aItem);
    aItem = new QStandardItem("");
    aItemList.append(aItem);
    msgModel->insertRow(msgModel->rowCount(),aItemList);

    aItemList.clear();
    aItem = new QStandardItem("生产单位");
    aItem->setEditable(false);
    aItem->setFont(font);
    aItem->setTextAlignment(Qt::AlignCenter);
    aItemList.append(aItem);
    aItem = new QStandardItem("");
    aItemList.append(aItem);
    msgModel->insertRow(msgModel->rowCount(),aItemList);

    connect(msgModel,SIGNAL(dataChanged(QModelIndex,QModelIndex)),this,SLOT(dataChangedHandle(QModelIndex,QModelIndex)),Qt::QueuedConnection);
    connect(carType,SIGNAL(currentTextChanged(QString)),this,SLOT(currentCarTypeChangedHandle(QString)),Qt::QueuedConnection);
    readCfg();

}

MsgInputWin::~MsgInputWin()
{
    saveCfg();
    delete ui;
}

void MsgInputWin::dataChangedHandle(const QModelIndex &topLeft, const QModelIndex &bottomRight)
{
    if(topLeft==bottomRight && bottomRight.row()==0){
        globalParams::company = msgModel->item(0,1)->text();
    }else if(topLeft==bottomRight && bottomRight.row()==1){
        globalParams::classGroup = msgModel->item(1,1)->text();
    }else if(topLeft==bottomRight && bottomRight.row()==3){
        globalParams::bdType = msgModel->item(3,1)->text();
    }else if(topLeft==bottomRight && bottomRight.row()==4){
        globalParams::bdSN = msgModel->item(4,1)->text();
    }else if(topLeft==bottomRight && bottomRight.row()==5){
        globalParams::marker = msgModel->item(5,1)->text();
    }
}

void MsgInputWin::currentCarTypeChangedHandle(const QString &text)
{
    globalParams::carType = text;
}

void MsgInputWin::readCfg()
{
    QSettings setting(globalParams::configPath,QSettings::IniFormat);
    QString company = setting.value("information/company").toString();
    QString group = setting.value("information/class_group").toString();
    QString curCarType = setting.value("information/current_car_type").toString();
    QString bdType = setting.value("information/bd_type").toString();
    QString bdSN = setting.value("information/bd_sn").toString();
    QString marker = setting.value("information/marker").toString();

    msgModel->item(0,1)->setText(company);
    msgModel->item(1,1)->setText(group);
    QComboBox *box = qobject_cast<QComboBox *>(ui->tableView->indexWidget(msgModel->index(2,1)));
    box->setCurrentText(curCarType);
    msgModel->item(3,1)->setText(bdType);
    msgModel->item(4,1)->setText(bdSN);
    msgModel->item(5,1)->setText(marker);
}

void MsgInputWin::saveCfg()
{
    QString company = msgModel->item(0,1)->text();
    QString group = msgModel->item(1,1)->text();

    QComboBox *box = qobject_cast<QComboBox *>(ui->tableView->indexWidget(msgModel->index(2,1)));

    QString curCarType = box->currentText();
    QString bdType = msgModel->item(3,1)->text();
    QString bdSN = msgModel->item(4,1)->text();
    QString marker = msgModel->item(5,1)->text();

    QSettings setting(globalParams::configPath,QSettings::IniFormat);
    setting.setValue("information/company",company);
    setting.setValue("information/class_group",group);
    setting.setValue("information/current_car_type",curCarType);
    setting.setValue("information/bd_type",bdType);
    setting.setValue("information/bd_sn",bdSN);
    setting.setValue("information/marker",marker);

}
