﻿#include "serialport.h"
#include <QDebug>

SerialPort::SerialPort(QObject *parent) : QObject(parent)
{
    serial = new QSerialPort;
    //thread = new QThread;
    //this->moveToThread(thread);
    //thread->start();
    connect(serial,SIGNAL(readyRead()),this,SLOT(readData()),Qt::QueuedConnection);
}

SerialPort::~SerialPort()
{
    serial->close();
    serial->deleteLater();
    //thread->quit();
    //thread->wait();
    //thread->deleteLater();
}

bool SerialPort::open()
{
    if(serial){
        serial->open(QIODevice::ReadWrite);
        if(serial->isOpen()){
            return true;
        }
    }
    return false;
}

void SerialPort::close()
{
    if(serial && serial->isOpen()){
        serial->close();
    }
}

void SerialPort::setSerialconfig(QString comName, int baud)
{
    if(serial){
        serial->setPortName(comName);
        serial->setBaudRate(baud);
        serial->setDataBits(QSerialPort::Data8);
        serial->setStopBits(QSerialPort::OneStop);
        serial->setParity(QSerialPort::NoParity);
        curBaud = baud;
    }
}

void SerialPort::setSerialconfig(int baud)
{
    if(serial){
        serial->setBaudRate(baud);
        curBaud = baud;
    }
}

bool SerialPort::isOpen()
{
    return serial->isOpen();
}

int SerialPort::getBaud()
{
    return curBaud;
}

void SerialPort::writeData(const QByteArray &buf)
{
    if(serial && serial->isOpen()){
        int size = serial->write(buf);
        writeBytesSignal(size);
    }
}

void SerialPort::readData()
{
    //qDebug()<<"子线程id"<<QThread::currentThreadId();
    if(serial){
        int size = serial->bytesAvailable();
        if(size<=0){
            return;
        }
        QByteArray buf(size,0);
        if(buf.data()){
            serial->read(buf.data(),size);
            emit recvSerialDataSignal(buf);
        }
    }
}
