﻿#include "sqlitedb.h"
#include <QDateTime>
#include <QMap>
#include <QDebug>

SQLITEDB::SQLITEDB(QObject *parent)
    :QObject(parent)
{
    thread = new QThread;
    this->moveToThread(thread);
    connect(thread, SIGNAL(finished()), this, SLOT(deleteLater()));
    thread->start();
    openDateBase();
}

SQLITEDB::~SQLITEDB()
{
    thread->quit();
    thread->wait();
    closeDateBase();
}

void SQLITEDB::openDateBase()
{
    DB=QSqlDatabase::addDatabase("QSQLITE");
    DB.setDatabaseName("database.db"); //设置数据库名称
    if (!DB.open())   //打开数据库
    {
        qDebug()<<"打开数据库失败";
        return;
    }else{
        qDebug()<<"打开数据库成功";
    }

    if(!tableExist("USERS")){
        QSqlQuery query(DB);
        query.prepare("CREATE TABLE [USERS](\
[ID] TEXT(255) NOT NULL UNIQUE,\
[PASSWORD] TEXT(4096),\
[USER_TYPE] TEXT(5));");

        if(!query.exec()){
            qDebug()<<"数据表创建失败";
        }else{
            //添加默认管理员账号
            query.prepare("INSERT INTO USERS (ID,PASSWORD,USER_TYPE) VALUES(:ID,:PASSWORD,:USER_TYPE);");
            query.bindValue(":ID","admin");
            query.bindValue(":PASSWORD","YWRtaW4=");
            query.bindValue(":USER_TYPE","Admin");
            if(!query.exec()){
                qDebug()<<"添加默认管理员账号失败";
            }
        }
    }

    if(!tableExist("INFORMATION")){
        QSqlQuery query(DB);
        query.prepare("CREATE TABLE [INFORMATION](\
[BDSN] TEXT(32) PRIMARY KEY NOT NULL, \
[TESTER] TEXT(255), \
[TEST_TIME] DATETIME, \
[COMPANY] TEXT(255), \
[GROUP] TEXT(255), \
[BD_TYPE] TEXT(255), \
[CAR_TYPE] TEXT(255), \
[MAKER] TEXT(255));");
    }

    if(!tableExist("INTERFACE_TEST")){
        QSqlQuery query(DB);
        query.prepare("CREATE TABLE [INTERFACE_TEST](\
[BDSN] TEXT(32) PRIMARY KEY NOT NULL, \
[TESTER] TEXT(255), \
[TEST_TIME] DATETIME, \
[SERIAL_SEND] TEXT(10), \
[SERIAL_RECV] TEXT(10), \
[CAN_SEND] TEXT(10), \
[CAN_RECV] TEXT(10));");
    }

    if(!tableExist("POSITION_PRECISION")){
        QSqlQuery query(DB);
        query.prepare("CREATE TABLE [POSITION_PRECISION](\
[BDSN] TEXT(32) NOT NULL, \
[TEST_TIME] DATETIME, \
[TESTER] TEXT(255), \
[MODE] TEXT(32) NOT NULL, \
[REF_LAT] TEXT(16), \
[REF_LON] TEXT(16), \
[SAMPLE_POINTS] TEXT(10), \
[HORIZONTAL_ERR] TEXT(10), \
[VERTICAL_ERR] TEXT(10));");
    }

    if(!tableExist("POSITION_TEST")){
        QSqlQuery query(DB);
        query.prepare("CREATE TABLE [POSITION_TEST](\
[BDSN] TEXT(32) NOT NULL, \
[TESTER] TEXT(255), \
[TEST_TIME] DATETIME, \
[MODE] TEXT(32) NOT NULL, \
[TIME] TEXT(32), \
[LONGITUDE] TEXT(16), \
[LATITUDE] TEXT(16), \
[HEIGHT] TEXT(16), \
[PDOP] TEXT(8), \[VDOP] TEXT(8), \
[HDOP] TEXT(8), \
[SAT_NUM] TEXT(8), \
PRIMARY KEY([BDSN], [MODE]));");
    }

    if(!tableExist("STATUS")){
        QSqlQuery query(DB);
        query.prepare("CREATE TABLE [STATUS](\
[BDSN] TEXT(32) PRIMARY KEY NOT NULL, \
[TESTER] TEXT(255), \
[TEST_TIME] DATETIME, \
[RNSS_SN] TEXT(32), \
[RDSS_SN] TEXT(32), \
[MPD009] TEXT(32), \
[MPD007] TEXT(32), \
[PRM] TEXT(32), \
[MPD007_VALID_DATE] TEXT(255), \
[PRM_VALID_DATE] TEXT(255), \
[BD_ID_NUM] TEXT(10), \
[BD_SERVICE_FEQ] TEXT(10), \
[BD_COMMUNICATION_LEVEL] TEXT(10), \
[BD_BROCAST_ADDRESS] TEXT(12), \
[BD_SECRECY_MARK] TEXT(10));");
    }

}

void SQLITEDB::closeDateBase()
{
    if(DB.isOpen()){
        DB.close();
    }
}

QSqlDatabase &SQLITEDB::getDateBase()
{
    return DB;
}

bool SQLITEDB::tableExist(const QString &table)
{
    QSqlQuery query(DB);
    query.prepare(QString("SELECT * FROM %1").arg(table));
    if(!query.exec()){
        qDebug()<<query.lastError().text();
        return false;
    }else{
        return true;
    }
}


bool SQLITEDB::keyExist(const QString &table, const QString &key)
{
    QSqlQuery query(DB);
    query.prepare(QString("SELECT %1 FROM %2").arg(key,table));
    if(!query.exec()){
        qDebug()<<query.lastError().text();
        return false;
    }else{
        return true;
    }
}

bool SQLITEDB::createKey(const QString &table, const QString &key, const QString &type)
{
    QSqlQuery query(DB);
    query.prepare(QString("ALTER TABLE %1 ADD %2 %3").arg(table,key,type));
    if(!query.exec()){
        qDebug()<<query.lastError().text();
        return false;
    }else{
        return true;
    }
}

bool SQLITEDB::createTable(const QString &sql)
{
    QSqlQuery query(DB);
    query.prepare(sql);
    if(!query.exec()){
        qDebug()<<query.lastError().text();
        return false;
    }else{
        return true;
    }
}

QStringList SQLITEDB::getPrimaryKeys(const QString &table)
{
    QStringList primaryKeys;
    QString cmd = QString("SELECT l.name FROM pragma_table_info(\"%1\") as l Where l.pk<>0;").arg(table);
    QSqlQuery query(DB);
    query.prepare(cmd);
    if(!query.exec()){
        qDebug()<<query.lastError().text();
    }
    while(query.next()){
        QString key = query.value("name").toString();
        if(!key.isEmpty()){
            primaryKeys.append(key);
        }
    }
    return primaryKeys;
}

QStringList SQLITEDB::getTableFields(const QString &table)
{
    QStringList keys;
    QString cmd = QString("SELECT l.name FROM pragma_table_info(\"%1\") as l;").arg(table);
    QSqlQuery query(DB);
    query.prepare(cmd);
    if(!query.exec()){
        qDebug()<<query.lastError().text();
    }
    while(query.next()){
        QString key = query.value("name").toString();
        if(!key.isEmpty()){
            keys.append(key);
        }
    }
    return keys;
}

QStringList SQLITEDB::getTableNames()
{
    QStringList tableNames;
    QString cmd = QString("SELECT name FROM sqlite_master WHERE type=\"table\" AND name!=\"sqlite_sequence\";");
    QSqlQuery query(DB);
    query.prepare(cmd);
    if(!query.exec()){
        qDebug()<<query.lastError().text();
    }
    while(query.next()){
        QString key = query.value("name").toString();
        if(!key.isEmpty()){
            tableNames.append(key);
        }
    }
    return tableNames;
}

QVector<QMap<QString, QString> > SQLITEDB::getRecord(const QString &table, const QMap<QString, QString> &condition)
{
    QVector<QMap<QString, QString> > vecMap;
    QSqlQuery query(DB);
    QString cmd = QString("SELECT * FROM %1 WHERE ").arg(table);
    QStringList keys;
    foreach (const QString &key, condition.keys()) {
        keys.append(key);
    }
    if(keys.size()>0){
        for(int i=0;i<keys.size();i++){
            cmd += QString("%1=:%2").arg(keys[i]).arg(keys[i]);
            if(i<keys.size()-1){
                cmd += " AND ";
            }
        }
    }
    query.prepare(cmd);
    for(int i=0;i<keys.size();i++){
        QString key = QString(":%1").arg(keys[i]);
        query.bindValue(key,condition.value(keys[i]));
    }
    query.exec();
    while(query.next()){
        QSqlRecord rec = query.record();
        QMap<QString, QString> map;
        int count = rec.count();
        for(int i=0;i<count;i++){
            QString key = rec.fieldName(i);
            QString value = rec.value(key).toString();
            map[key] = value;
        }
        vecMap.append(map);
    }

    return vecMap;
}

bool SQLITEDB::saveRecord(const QString &table, const QMap<QString, QString> &map)
{
    QStringList primaryKeys = SQLITEDB::getInstance().getPrimaryKeys(table);

    QStringList tableFields = SQLITEDB::getInstance().getTableFields(table);
    QSet<QString> fields;
    for(int i=0;i<tableFields.size();i++){
        fields.insert(tableFields[i]);
    }

    //查询数据库中是否存在该记录
    QSqlQuery query(SQLITEDB::getInstance().getDateBase());
    QString cmd = QString("SELECT * FROM %1").arg(table);
    QString condition = " WHERE ";
    if(primaryKeys.size()>0){
        for(int i=0;i<primaryKeys.size();i++){
            condition += QString("%1=:%2").arg(primaryKeys[i]).arg(primaryKeys[i]);
            if(i<primaryKeys.size()-1){
                condition += " AND ";
            }
        }
        cmd += condition;
    }

    query.prepare(cmd);
    for(int i=0;i<primaryKeys.size();i++){
        QString key = QString(":%1").arg(primaryKeys[i]);
        QString value = map.value(primaryKeys[i]," ");
        query.bindValue(key,value);
    }

    query.exec();
    if(query.first()){
        QString val;
        foreach (const QString &key, map.keys()) {
            if(!fields.contains(key)){
                continue;
            }
            val += QString("%1=:%2,").arg(key,key);
        }
        val = val.left(val.lastIndexOf(","));//去掉最后一个","
        QString command = QString("UPDATE %1 set %2 %3;").arg(table,val,condition);
        query.prepare(command);
    }else{//无记录插入
        QString key,val;
        foreach (const QString &_key, map.keys()) {
            if(!fields.contains(_key)){
                continue;
            }
            key += QString("%1,").arg(_key);
            val += QString(":%1,").arg(_key);
        }
        key = key.left(key.lastIndexOf(","));//去掉最后一个","
        val = val.left(val.lastIndexOf(","));//去掉最后一个","
        QString command = QString("INSERT INTO %1 (%2) VALUES(%3);").arg(table,key,val);
        query.prepare(command);
    }
    foreach (const QString &_key, map.keys()) {
        if(!fields.contains(_key)){
            continue;
        }
        QString key = QString(":%1").arg(_key);
        query.bindValue(key,map.value(_key));
    }
    if(!query.exec()){
        qDebug()<<query.lastError().text();
        return false;
    }
    return true;
}



