﻿#include "titlebar.h"
#include "ui_titlebar.h"
#include <QLabel>
#include <QPushButton>
#include <qt_windows.h>
#include <QDebug>

#pragma comment(lib, "User32.lib")

TitleBar::TitleBar(QWidget *parent) :
    QWidget(parent),
    ui(new Ui::TitleBar)
{
    ui->setupUi(this);
    setAttribute(Qt::WA_StyledBackground);
    //this->setPalette(QPalette(QColor(235,246,249)));
    this->setPalette(QPalette(QColor(240,240,240)));

    this->setAutoFillBackground(true);
    ui->btnMin->setStyleSheet("QPushButton{background-image:url(:/images/min.png);\
                               background-color:rgb(240,240,240);background-position:center;border:none}" \
                               "QPushButton:hover{background-color:rgba(233,233,233,150);" \
                                "background-image:url(:/images/min.png);border:none;}");
    ui->btnMax->setStyleSheet("QPushButton{background-image:url(:/images/normal.png);\
                              background-color:rgb(240,240,240);background-position:center;border:none}" \
                              "QPushButton:hover{background-color:rgba(233,233,233,150);}");
    ui->btnClose->setStyleSheet("QPushButton{background-image:url(:/images/close.png);\
                              background-color:rgb(240,240,240);background-position:center;border:none}" \
                              "QPushButton:hover{background-image:url(:/images/close_hover.png);\
                               background-color:rgba(196,43,28,220);}");
    ui->btnMax->setVisible(false);
    connect(ui->btnMin,SIGNAL(clicked(bool)),this,SLOT(btnClickHandle()),Qt::QueuedConnection);
    connect(ui->btnMax,SIGNAL(clicked(bool)),this,SLOT(btnClickHandle()),Qt::QueuedConnection);
    connect(ui->btnClose,SIGNAL(clicked(bool)),this,SLOT(btnClickHandle()),Qt::QueuedConnection);
}

TitleBar::~TitleBar()
{
    delete ui;
}

void TitleBar::mousePressEvent(QMouseEvent *event)
{
    Q_UNUSED(event);
    if(ReleaseCapture()){
        QWidget* pWindow = this->window();
        if(pWindow->isTopLevel()){
            SendMessage(HWND(pWindow->winId()), WM_SYSCOMMAND, SC_MOVE + HTCAPTION, 0);
        }
    }
}

void TitleBar::mouseDoubleClickEvent(QMouseEvent *event)
{
    Q_UNUSED(event);
    emit ui->btnMax->clicked();
}

void TitleBar::btnClickHandle()
{

    QPushButton *btn = qobject_cast<QPushButton*>(sender());
    QWidget *pWindow = this->window();
    if(btn == ui->btnMin){
         pWindow->showMinimized();
    }else if(btn == ui->btnMax){
        if(pWindow->isMaximized()){
             pWindow->showNormal();
             ui->btnMax->setStyleSheet("QPushButton{background-image:url(:/images/normal.png);\
                                       background-color:rgb(255,255,255);background-position:center;border:none}" \
                                       "QPushButton:hover{background-color:rgba(233,233,233,150);}");
         }else{
            pWindow->showMaximized();
            ui->btnMax->setStyleSheet("QPushButton{background-image:url(:/images/max.png);\
                                       background-color:rgb(255,255,255);background-position:center;border:none}" \
                                       "QPushButton:hover{background-color:rgba(233,233,233,150);}");
         }
    }else if(btn == ui->btnClose){
        emit windowCloseSignal();
    }
}
